/********************************************/
/*                                    	    */
/*   PKTC.H  Version 1.00           	    */
/*                                    	    */
/*   Header file for Packet Driver library  */
/*                                    	    */
/*   Libor Hlavac, (c) 1993                 */
/*                                    	    */
/********************************************/

#if __STDC__
#define _Cdecl
#else
#define _Cdecl	cdecl
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define E_BAD_HANDLE	  1
#define E_NO_CLASS	  2
#define E_NO_TYPE   	  3
#define E_NO_NUMBER       4
#define E_BAD_TYPE	  5
#define E_NO_MULTICAST	  6
#define E_CANT_TERMINATE  7
#define E_BAD_MODE	  8
#define E_NO_SPACE	  9
#define E_TYPE_INUSE	 10
#define E_BAD_COMMAND	 11
#define E_CANT_SEND	 12
#define E_CANT_SET	 13
#define E_BAD_ADDRESS	 14
#define E_CANT_RESET	 15
#define E_NO_PACKET	 16
#define E_NOT_INIT	 17
#define E_NO_DRIVER	 18

char * error_str[] = {"Ok.",
		      "Bad handle",
		      "No class",
		      "No type",
		      "No number",
		      "Bad type",
		      "No multicast",
		      "Can't terminate",
		      "Bad mode",
		      "No space",
		      "Type in use",
		      "Bad command",
		      "Can't send",
		      "Can't set",
		      "Bad address",
		      "Can't reset",
		      "No packet",
		      "Not initialized",
		      "No packet driver found" };


#define ADDRLEN   6
#define HEADERLEN 14
#define MAXDATALEN 1500

typedef unsigned char uchar;

#define packet struct _packet

struct _packet
{
  uchar    dest  [ADDRLEN];
  uchar    source[ADDRLEN];
  unsigned type;
  uchar    data  [MAXDATALEN];
};


/* --------------------------------------------------------------------------
    write_addr - writes address to stdout
		 argument:
		 - addr = pointer to address buffer                        */

void write_addr (uchar * addr);

/* --------------------------------------------------------------------------
    write_hex - writes bytes (hex)
		argument:
		- c = byte                                                 */

void write_hex (uchar c);

/* -------------------------------------------------------------------------
    pkt_init - Packet Driver inicialization
	       arguments:
	       - type   = packet type (0 - 0xFFFF)

	       - handle = pointer to store new handle

	       result:
	       - 0           = success
	       - error_code                                               */

int far _Cdecl pkt_init (unsigned type, unsigned far * handle);

/* -------------------------------------------------------------------------
    pkt_get_addr - Get Packet Driver address
		   arguments:
		   - handle = your handle
		   - buffer = pointer to six bytes long buffer for address

		   result:
		   - 0           = success
		   - error_code                                           */

int  far _Cdecl pkt_get_addr (unsigned handle, uchar far * buffer);

/* -------------------------------------------------------------------------
    pkt_send - Send packet
	       arguments:
	       - buffer = packet to send
	       - length = packet length

	       result:
	       - 0           = success
	       - error_code                                               */

int far _Cdecl pkt_send (packet far * buffer, unsigned length);

/* -------------------------------------------------------------------------
    pkt_rec - Receive packet
	      arguments:
	      - buffer = buffer for incoming packet
	      - length = buffer for incoming packet legnth

	      result:
	      - 0           = success
	      - error_code                                                */

int far _Cdecl pkt_rec (packet far * buffer, unsigned far * length);

/* -------------------------------------------------------------------------
    pkt_close - release packet type
		arguments:
		- handle = your handle

		result:
		- 0           = success
		- error_code                                              */


int far _Cdecl pkt_close (unsigned handle);

/* ---------------------------------------------------------------------- */

#ifdef __cplusplus
}
#endif

/**************************************************************************/