;********************************************
;*                                    	    *
;*   PKTCA.ASM  Version 1.00           	    *
;*                                    	    *
;*   Packet Driver <=> C library	    *
;*                                    	    *
;*   Libor Hlavac, (c) 1993                 *
;*                                    	    *
;********************************************

masm51
jumps
.286

; **********  Packet Driver error codes  **********

E_BAD_HANDLE	equ	1
E_NO_CLASS	equ	2
E_NO_TYPE   	equ	3
E_NO_NUMBER	equ	4
E_BAD_TYPE	equ	5
E_NO_MULTICAST	equ	6
E_CANT_TERMINATE equ	7
E_BAD_MODE	equ	8
E_NO_SPACE	equ	9
E_TYPE_INUSE	equ	10
E_BAD_COMMAND	equ	11
E_CANT_SEND	equ	12
E_CANT_SET	equ	13
E_BAD_ADDRESS	equ	14
E_CANT_RESET	equ	15

; **********  Library error codes  **********

E_NO_PACKET	equ	16
E_NOT_INIT	equ	17
E_NO_DRIVER	equ	18

; **********  Packet Driver service numbers  **********

S_DRIVER_INFO	equ	1
S_ACCESS_TYPE	equ	2
S_RELEASE_TYPE	equ	3
S_SEND_PKT	equ	4
S_TERMINATE	equ	5
S_GET_ADDRESS	equ	6
S_RESET_IFACE	equ	7

; **********  Other library constants  **********

ANYTYPE		equ	0FFFFh
TYPELEN		equ	2
BUFLEN		equ	1520
NUMBUFF		equ	4
ADDRLEN		equ	6
LENLEN		equ	2

; **********  C data segments  **********

DGROUP		group	_DATA, _BSS
_DATA		segment	word public 'DATA'
_DATA		ends
_BSS		segment word public 'BSS'
_BSS		ends

; **********  Packet Driver call macro  **********

pkt_int		macro
		pushf
		cli
		call	cs:driver
		endm

; **********  string fill macro  **********

dfill    	macro   fill,flen
		local 	block,rest,len,string
  string	equ	fill
  len           sizestr string
  len		=	len - 2
  block		=	flen /   len
  rest		=       flen mod len
  rept block
		db	fill
  endm
  rept rest
		db 	'#'
  endm
endm

; **********  code segment  **********

_TEXT		segment byte public 'CODE'
		assume	cs:_TEXT, ds:DGROUP

; **********  library variables  **********

driver		dd	0
sign		db	'PKT DRVR'
sign_len	equ	$ - sign
buffbusy	db	?
nextread	db	?
nextwrite	db	?

; **********  library buffers  **********

buffers		label	byte
		dfill   'Buffer 1',BUFLEN
		dfill   'Buffer 2',BUFLEN
		dfill   'Buffer 3',BUFLEN
		dfill   'Buffer 4',BUFLEN

; **********  _pkt_init  **********

		public	_pkt_init

_pkt_init	proc	far
		push	bp
		mov	bp, sp
		arg	atype : word, phandle : dword
		push	si di ds
		mov	ax, cs
		mov	ds, ax

		xor	al, al
		mov	buffbusy, al
		mov	nextread, al
		mov	nextwrite, al

		call	find_int
		call	test_err
		jc	init_done

		mov	ax, (S_DRIVER_INFO SHL 8) + 0FFh
		pkt_int
		call	test_err
		jc	init_done

		mov	ax, cs
		mov	es, ax
		mov	ah, S_ACCESS_TYPE
		mov	al, ch
		mov	bx, ANYTYPE
		mov	dl, cl
		lea	si, atype
		push	ss
		pop	ds
		mov	cx, TYPELEN
		lea	di, receiver
		pkt_int
		call	test_err
		jc	init_done

		les	bx, phandle
		mov	es:[bx], ax
		xor	ax, ax
init_done:	pop	ds di si
		pop	bp
		ret
_pkt_init	endp

; **********  _pkt_get_addr  **********

		public	_pkt_get_addr

_pkt_get_addr	proc	far
		push	bp
		mov	bp, sp
		arg 	handle : word, buffer : dword
		push	si di ds
		call	test_int
		call	test_err
		jc	get_done

		mov	ah, S_GET_ADDRESS
		mov	bx, handle
		les	di, buffer
		mov	cx, ADDRLEN
		pkt_int
		call	test_err
		jc	get_done
		xor	ax, ax
get_done:	pop	ds di si
		pop	bp
		ret
_pkt_get_addr	endp

; **********  _pkt_send  **********

		public	_pkt_send

_pkt_send	proc	far
		push	bp
		mov	bp, sp
		arg	buffer : dword, len : word
		push	si di ds
		call	test_int
		call	test_err
		jc	send_done

		mov	ah, S_SEND_PKT
		lds	si, buffer
		mov	cx, len
		pkt_int
		call	test_err
		jc	send_done
		xor	ax, ax
send_done:	pop	ds di si
		pop 	bp
		ret
_pkt_send	endp

; **********  _pkt_close  **********

		public	_pkt_close

_pkt_close	proc	far
		push	bp
		mov	bp, sp
		arg	handle : word
		push	si di ds
		call	test_int
		call	test_err
		jc	close_done

		mov	ah, S_RELEASE_TYPE
		mov	bx, handle
		pkt_int
		call	test_err
		jc	close_done
		xor	ax, ax
close_done:     pop	ds di si
		pop 	bp
		ret
_pkt_close	endp

; **********  _pkt_rec  **********

		public	_pkt_rec

_pkt_rec	proc	far
		push	bp
		mov	bp, sp
		arg     buff : dword, len : dword
		push	si di ds
		xor	ch, ch
		mov	cl, cs:nextread
		mov	ah, 1
		shl	ah, cl
		mov	bl, NUMBUFF

look_loop:	mov	al, cs:buffbusy
		and	al, ah
		jz	buff_empty

		mov	al, cl
		inc	al
		and	al, NUMBUFF - 1
		mov	cs:nextread, al

		not	ah
		and	cs:buffbusy, ah

		lea	si, buffers
		jcxz	add_done
add_loop:	add	si, BUFLEN
		loop	add_loop
add_done:       mov	ax, cs
		mov	ds, ax
		mov	cx, [si]
		add	si, LENLEN
		les	di, len
		mov	es:[di], cx
		les	di, buff
		rep	movsb
		xor	ax, ax
		jmp	rec_done

buff_empty:	inc	cl
		and	cl, NUMBUFF - 1

		shl	ah, 1
		cmp	ah, (1 SHL (NUMBUFF - 1))
		jna	mask2_ok
		mov	ah, 1
mask2_ok:       dec	bl
		jnz	look_loop
		mov	ax, E_NO_PACKET

rec_done:	pop	ds di si
		pop	bp
		ret
_pkt_rec	endp

; **********  receiver  **********

receiver	proc	far
		push	bx cx dx
		cmp	ax, 0
		jne	pkt_copied

buff_request:	cmp	cx, BUFLEN - 2
		ja	rec_reject
		mov	dx, cx
		xor	ch, ch
		mov	cl, cs:nextwrite
		mov	ah, 1
		shl	ah, cl
		mov	bl, NUMBUFF

look_for_buff:	mov	al, cs:buffbusy
		and	al, ah
		jz	put_buff
		inc	cl
		and	cl, NUMBUFF - 1
		shl	ah, 1
		cmp	ah, (1 SHL (NUMBUFF - 1))
		jna	mask1_ok
		mov	ah, 1
mask1_ok:       dec	bl
		jnz	look_for_buff

rec_reject:     xor	ax, ax
		mov	es, ax
		mov	di, ax
		jmp	receiver_end

put_buff:       mov	al, cl
		inc 	al
		and	al, NUMBUFF - 1
		mov	cs:nextwrite, al
		lea	di, buffers
		jcxz	sum_done
sum_loop:       add	di, BUFLEN
		loop	sum_loop
sum_done:       mov	ax, cs
		mov	es, ax
		mov	es:[di], dx
		add	di, LENLEN
		jmp	receiver_end

pkt_copied:	mov	ax, si
		sub	ax, offset buffers + LENLEN
		xor	dx, dx
		mov	bx, BUFLEN
		div	bx
		cmp	dx, 0
		jne	copy_err
		cmp	ax, NUMBUFF - 1
		ja	copy_err
		mov	cx, ax
		mov	ah, 1
		shl	ah, cl
		or	cs:buffbusy, ah
copy_err:
receiver_end:	pop	dx cx bx
		ret
receiver	endp

; **********  find_int  **********

find_int	proc	near                    ; vyhledani paket driveru
		mov	dx, 60h                 ; 60h - 80h

find_loop:	mov	al, dl
		mov	ah, 35h
		int	21h
		mov     word ptr cs:driver    , bx ; adresy obsluh
		mov	word ptr cs:driver + 2, es
		mov	di, bx
		add	di, 3
		lea	si, sign
		mov	cx, sign_len
		repe    cmpsb                   ; porovnej singaturu
		je	found_int
		inc	dl
		cmp	dl, 80h
		jc	find_loop
		xor	ax,ax
		mov	word ptr cs:driver, ax
		mov	word ptr cs:driver + 2, ax
		mov	dh, E_NO_DRIVER
		stc
		ret

found_int:	clc
		ret
find_int	endp

; **********  test_int  **********

test_int	proc	near
		mov	ax, word ptr cs:driver
		or	ax, word ptr cs:driver + 2
		clc
		jnz	int_ok
		mov	dh, E_NOT_INIT
		stc
int_ok:		ret
test_int	endp

; **********  test_err  **********

test_err	proc	near
		jnc	test_done
		mov	al, dh
		mov	ah, 0
test_done:	ret
test_err	endp

_TEXT		ends

; ***************************************************************************

		end
