{*********************************************}
{*                                           *}
{*   PKTPAS.PAS  Version 1.00                *}
{*                                           *}
{*   Packet Driver <=> Turbo PASCAL library  *}
{*                                           *}
{*   Libor Hlavac, (c) 1993                  *}
{*                                           *}
{*********************************************}

unit PktPas;

interface

  const
    ADDRLEN    = 6;
    MAXDATALEN = 1500;
    HEADERLEN  = 14;

    E_BAD_HANDLE     = 1;
    E_NO_CLASS	     = 2;
    E_NO_TYPE        = 3;
    E_NO_NUMBER      = 4;
    E_BAD_TYPE	     = 5;
    E_NO_MULTICAST   = 6;
    E_CANT_TERMINATE = 7;
    E_BAD_MODE	     = 8;
    E_NO_SPACE	     = 9;
    E_TYPE_INUSE     = 10;
    E_BAD_COMMAND    = 11;
    E_CANT_SEND	     = 12;
    E_CANT_SET	     = 13;
    E_BAD_ADDRESS    = 14;
    E_CANT_RESET     = 15;
    E_NO_PACKET	     = 16;
    E_NOT_INIT	     = 17;
    E_NO_DRIVER      = 18;

    ErrorStr : array [0..18] of string = (
                      'Ok.',
		      'Bad handle',
		      'No class',
		      'No type',
		      'No number',
		      'Bad type',
		      'No multicast',
		      'Can''t terminate',
		      'Bad mode',
		      'No space',
		      'Type in use',
		      'Bad command',
		      'Can''t send',
		      'Can''t set',
		      'Bad address',
		      'Can''t reset',
		      'No packet',
		      'Not initialized',
		      'No packet driver found' );


  type
    THandle = word;
    TType   = word;
    TAddr   = array [1..ADDRLEN] of byte;
    TPacket = record
                Dest   : TAddr;
                Source : TAddr;
                Typ    : word;
		Data   : array [1..MAXDATALEN] of byte;
	      end;

{----------------------------------------------------------------------------
  PktInit - Packet Driver inicialization
	    arguments:
	    - Typ    = packet type (0 - 0xFFFF)
	    - Handle = function returns Handle

	    result:
	    - 0           = success
	    - error_code                                                    }

  function PktInit    (Typ : TType; var Handle : THandle)    : integer;

{----------------------------------------------------------------------------
  PktGetAddr - Get Packet Driver address
	       arguments:
	       - Handle = your handle
	       - Addr   = function returns address

	       result:
	       - 0           = success
	       - error_code                                                 }

  function PktGetAddr (Handle : THandle; var Addr : TAddr)   : integer;

{----------------------------------------------------------------------------
  PktSend - Send packet
	    arguments:
	    - Packet = packet to send
	    - Len    = packet length

	    result:
	    - 0           = success
	    - error_code                                                    }

  function PktSend    (var Packet : TPacket; Len : word)     : integer;

{----------------------------------------------------------------------------
  PktRec - Receive packet
	   arguments:
	   - Packet = empty packet
	   - Len    = functions returns packet length

	   result:
	   - 0           = success
	   - error_code                                                     }


  function PktRec     (var Packet : TPacket; var Len : word) : integer;

{----------------------------------------------------------------------------
  PktClose - release packet type
	     arguments:
	     - Handle = your handle

	     result:
	     - 0           = success
	     - error_code                                                   }

  function PktClose   (Handle : THandle)                     : integer;

{----------------------------------------------------------------------------
  WriteHex - write hex number
	     arguments:
	     - c = byte to write                                            }

  procedure WriteHex (c:byte);

{----------------------------------------------------------------------------
  WriteAddr - write address
	      arguments:
	      - Addr = Address                                              }

  procedure WriteAddr (Addr : TAddr);

{---------------------------------------------------------------------------}


implementation

  function PktInit;    external;
  function PktGetAddr; external;
  function PktSend;    external;
  function PktRec;     external;
  function PktClose;   external;

  {$L PKTPASA.OBJ}

  procedure WriteHex;
  const HexTab : array[0..15] of char = '0123456789ABCDEF';

  begin
    Write(HexTab[(c SHR 4) AND $F]);
    Write(HexTab[c AND $F])
  end;

  procedure WriteAddr;
  var i:byte;
  begin
    for i:=1 to 6 do WriteHex(Addr[i])
  end;
end.

{---------------------------------------------------------------------------}
