///////////////////////////////////////////////////////////////////
//
// ip-arp packed test = ARP-QUERY
//
// Popis: Program vytvori IP/ARP-QUERY paket podle zadanych parametru
//	  a vysle ho jako broadcast do lokalni site.
//	  Stanice s pozadovanym ip pak zasila odpoved (IP/ARP-RESPONSE).
//
// Literatura: Popis PktDrv
//
// Dne:	  23.4.1997 	- Martin Cerveny - prvni verze
//	  24.4.1997	- Martin Cerveny - upraveno pro skolu
//
///////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <dos.h>
#include <string.h>

#include "pktdrvr.h"				// funkce pktdrv

// --------------------- PARAMETRY ++++ NUTNE UPRAVIT ++++

#define	PKT_INT		0x69			// kde se hleda pktdrv
#define	SOURCE_IP	"192.108.159.x"		// moje ip adresa
#define	TARGET_IP	"192.108.159.y"		// vzdalena adresa

// --------------------- STRUKTURY


// --- ETH-II hlavicka

typedef struct tagETHERNET {			// hlacvicka ETH
	unsigned char dst_addr[6];
	unsigned char src_addr[6];
	unsigned int type;			// ramcovani EthII (DIX)
	} ETHERNET;

#define ETH_ARP		0x0806			// ARP typ
#define ETH_IP		0x0800			// IP typ
#define ETH_IPX		0x8192			// NovellIPX typ

// --- ARP data pro ethernet

typedef struct tagARP_ETH {			// Struktura ARP pro Ethernet
	unsigned int	hardware;		// ARP_ETH_HW
	unsigned int	protocol;		// ARP_PROTOCOL_IP
	unsigned char	LHA;			// ARP_LHA
	unsigned char	LPA;			// ARP_LPA
	unsigned int	opcode;			// ARP_REQUEST/ARP_REPLY
	unsigned char	SHA[6], SPA[4];
	unsigned char	THA[6], TPA[4];
	} ARP_ETH;

#define ARP_ETH_HW	0x0001
#define	ARP_PROTOCOL_IP	ETH_IP
#define	ARP_LHA		6
#define	ARP_LPA		4
#define ARP_REQUEST	0x0001
#define ARP_REPLY	0x0002

// --- packet ARP nad Ethernetem

typedef struct tagPACKET {
	ETHERNET header;
	ARP_ETH	 arp;
	} PACKET;

// ----------------------------- DATA

char    pktrcv[1514];				// buffer pro prijem (bez FCS)
int	ready=0;				// priznak konce

// ----------------------------- PROCEDURY

int cpyip(char *str, char *ip)			// prevod znakove ip adresy
{						// na cislo
  int i[4];

  if (sscanf(str, "%i.%i.%i.%i", i, i+1, i+2, i+3)==4) {
    ip[0]=i[0]; ip[1]=i[1]; ip[2]=i[2]; ip[3]=i[3];
    return 1;
  }
  return 0;					// chyba pri prevodu
}


int test_for_pd(unsigned int intno)		// test, zda je pritomny
{						// packetdriver
  long drvvec;

  drvvec = (long)getvect(intno);		// musi existovat zadany text
  return !_fstrncmp(((char _far *)drvvec)+3,"PKT DRVR",8);
}


void _far _saveregs _loadds receiver()		// call-back funkce pro ARP
{						// protokol

  switch (((saveregs_access *)MK_FP(_SS, _BP))->ax) {
						// prvni vstup -> vracime buffer
    case 0:					// test velikosti paketu
      if (((saveregs_access *)MK_FP(_SS, _BP))->cx<sizeof(pktrcv)) {
	((saveregs_access *)MK_FP(_SS, _BP))->es=FP_SEG(&pktrcv);
	((saveregs_access *)MK_FP(_SS, _BP))->di=FP_OFF(&pktrcv);
      }
      else {					// neni dostatek mista pro
	((saveregs_access *)MK_FP(_SS, _BP))->es= // paket -> vracime 0:0
	 ((saveregs_access *)MK_FP(_SS, _BP))->di=0;
      }
      break;

    case 1:					// paket prekopirovan
      ready=1;					// do bufferu (ds:si)
      break;
  }
}

// -------------------------------- HLAVNI FUNKCE

int main()
{
  int 	 	if_class, if_type, if_number;	// informace o pktdrv
  int 	 	handle;				// odkaz na vazbu L2/L3
  unsigned int 	type=IntSwap(ETH_ARP);		// typ paketu L3
  PACKET 	pkt;				// paket pro odeslani
  int		pkt_int=PKT_INT;		// kde je paketdriver ?

  if (test_for_pd(pkt_int)) {			// testujeme a zjistujeme
						// info o pktdrv
    driver_info(pkt_int, 255, NULL, &if_class, &if_type, &if_number, NULL);
						// (jeste je mozne otestovat
						//  navratovy kod !)
    printf("PacketDriver: Class %i, Type %i, Number %i\n",
	   if_class, if_type, if_number);
						// sestaveni ARP-dotazu
						//  ... nejdrive ETH hlavicka
    memset(pkt.header.dst_addr, 0xFF, sizeof(pkt.header.dst_addr));
    get_address(pkt_int, 255, pkt.header.src_addr, sizeof(pkt.header.src_addr));
    pkt.header.type=IntSwap(ETH_ARP);
						//  ... pak ARP data
    pkt.arp.hardware=IntSwap(ARP_ETH_HW);
    pkt.arp.protocol=IntSwap(ARP_PROTOCOL_IP);
    pkt.arp.LHA=ARP_LHA;
    pkt.arp.LPA=ARP_LPA;
    pkt.arp.opcode=IntSwap(ARP_REQUEST);
    memcpy(pkt.arp.SHA, pkt.header.src_addr, sizeof(pkt.arp.SHA));
    if (!cpyip(SOURCE_IP, pkt.arp.SPA)) {
       printf("Chyba ve zdrojove IP adrese\n");
       return(2);
    }
    memset(pkt.arp.THA, 0, sizeof(pkt.arp.THA));
    if (!cpyip(TARGET_IP, pkt.arp.TPA)) {
       printf("Chyba ve cilove IP adrese\n");
       return(3);
    }
						// pripravime se na prijem
    handle=access_type(pkt_int, if_class, if_type, if_number,
		      (char *)&type, sizeof(type), receiver);

						// odesleme dotaz
    send_pkt(pkt_int, (char *)&pkt, sizeof(pkt));

    while (!ready && !kbhit());			// cekame na odpoved
						// (treba dopnit TO)
						// (pripadne i opakovani dotazu!)

    release_type(pkt_int, handle);		// a vypiseme

    if (ready)
      printf("Hardwarova adresa pro %s je: %02X:%02X:%02X:%02X:%02X:%02X\n",
        TARGET_IP,
        ((PACKET *)pktrcv)->arp.SHA[0], ((PACKET *)pktrcv)->arp.SHA[1],
        ((PACKET *)pktrcv)->arp.SHA[2], ((PACKET *)pktrcv)->arp.SHA[3],
        ((PACKET *)pktrcv)->arp.SHA[4], ((PACKET *)pktrcv)->arp.SHA[5]);
    else
      printf("Funkce prerusena\n");
  }
  else {
    printf("Neni instalovan packet driver !\n");
    return(1);
  }

  return 0;
}
